# Copyright 2018 Intel Corporation.
# The source code, information and material ("Material") contained herein is
# owned by Intel Corporation or its suppliers or licensors, and title to such
# Material remains with Intel Corporation or its suppliers or licensors.
# The Material contains proprietary information of Intel or its suppliers and
# licensors. The Material is protected by worldwide copyright laws and treaty
# provisions.
# No part of the Material may be used, copied, reproduced, modified, published,
# uploaded, posted, transmitted, distributed or disclosed in any way without
# Intel's prior express written permission. No license under any patent,
# copyright or other intellectual property rights in the Material is granted to
# or conferred upon you, either expressly, by implication, inducement, estoppel
# or otherwise.
# Any license under such intellectual property rights must be express and
# approved by Intel in writing.


from .Layer import Layer
from Controllers.TensorFormat import LayerIOFormat, LayerIOFormatPair
import Models.Layouts as Layouts
import numpy as np

class Crop(Layer):
    _supported_IO_formats = [
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHWC_to_NHWC_compact),
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NCHW_to_NCHW_fullstride),
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHCW_to_NHCW_fullstride),
        ]

    def __init__(self, *args):
        super().__init__(*args)

        # Default to no offset on any axis.
        self.crop_offset = np.array([0, 0, 0], np.dtype("<u4"))

    def __repr__(self):
        repr = super().__repr__()
        repr += "\n  [Crop specific]:\n"
        repr += "      offsets: %r\n" % (self.crop_offset)
        return repr

    def getIOFormats(self):
        # This is a hack to address the following problem:
        # The implicit Crop is implemented by placing it's output tensor inside the input tensor.
        # But, since the Crop's input/output tensors get their enclosures selected depending
        # on the Crop's input and output, separately, the two calculated enclosures might
        # be different, which is definitely bad for the implicit placing
        # This hack assumes that the implicit Crop is always followed by a HwConvolution,
        # hence we set the enclosure request of the Crop to match the one of the
        # subsequent HwConvolution
        if self.getImplicit():
            self._supported_IO_formats = [
                LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHCW_to_NHCW_C_stride_8),
                LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NCHW_to_NCHW_H_stride_8),
                ]
        else:
            self._supported_IO_formats = [
                LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHWC_to_NHWC_compact),
                LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NCHW_to_NCHW_fullstride),
                LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHCW_to_NHCW_fullstride),
            ]
        return super().getIOFormats()

    def setOffset(self, arr):
        assert len(arr) == 3
        self.crop_offset = arr

    def getOffset(self):
        return self.crop_offset
