# Copyright 2018 Intel Corporation.
# The source code, information and material ("Material") contained herein is
# owned by Intel Corporation or its suppliers or licensors, and title to such
# Material remains with Intel Corporation or its suppliers or licensors.
# The Material contains proprietary information of Intel or its suppliers and
# licensors. The Material is protected by worldwide copyright laws and treaty
# provisions.
# No part of the Material may be used, copied, reproduced, modified, published,
# uploaded, posted, transmitted, distributed or disclosed in any way without
# Intel's prior express written permission. No license under any patent,
# copyright or other intellectual property rights in the Material is granted to
# or conferred upon you, either expressly, by implication, inducement, estoppel
# or otherwise.
# Any license under such intellectual property rights must be express and
# approved by Intel in writing.


from .Layer import Layer
import Models.Layouts as Layouts
from Controllers.TensorFormat import LayerIOFormat, LayerIOFormatPair

class ReLU(Layer):
    _supported_IO_formats = [
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHWC_to_NHWC_W_stride),
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHCW_to_NHCW_C_stride)
        ]

    def __init__(self, *args):
        super().__init__(*args)
        self.reluX = 0

        self.negativeSlope = 0.0

    def loadReluX(self, X):
        self.reluX = X

    def loadNegativeSlope(self, nSlope):
        self.negativeSlope = nSlope

class LeakyReLU(ReLU):

    def loadNegativeSlope(self, nSlope):
        self.negativeSlope = nSlope

class ReLUWithBias(ReLU):

    _supported_IO_formats = [
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHWC_to_NHWC_W_stride),
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHCW_to_NHCW_C_stride)
        ]

    def __init__(self, base):
        super().__init__(base.getStringifiedName(), base.inputTensorNames, base.outputTensorNames)

        # Copy the attributes from the parent object
        for attr_name in base.__dict__:
            setattr(self, attr_name, getattr(base, attr_name))
        self.bias = None

    def loadBias(self, bias):
        self.bias = bias

    def getBias(self):
        return self.bias