#!/usr/bin/env python3

# Copyright 2018 Intel Corporation.
# The source code, information and material ("Material") contained herein is
# owned by Intel Corporation or its suppliers or licensors, and title to such
# Material remains with Intel Corporation or its suppliers or licensors.
# The Material contains proprietary information of Intel or its suppliers and
# licensors. The Material is protected by worldwide copyright laws and treaty
# provisions.
# No part of the Material may be used, copied, reproduced, modified, published,
# uploaded, posted, transmitted, distributed or disclosed in any way without
# Intel's prior express written permission. No license under any patent,
# copyright or other intellectual property rights in the Material is granted to
# or conferred upon you, either expressly, by implication, inducement, estoppel
# or otherwise.
# Any license under such intellectual property rights must be express and
# approved by Intel in writing.

import numpy as np
from Controllers.Parsers.Parser.Scale import Scale
from Controllers.Parsers.TensorFlowParser.Helpers import getInputNames, getOutputNames, findTensorValue

def load(obj, operations, feed_dict):
    inpts = getInputNames(obj)

    x = Scale(obj.name, [inpts[0]], [getOutputNames(obj)[0]])

    eps = obj.get_attr('epsilon')

    scale_param = findTensorValue(inpts[1], operations)
    offset = findTensorValue(inpts[2], operations)
    mean = findTensorValue(inpts[3], operations)
    var = findTensorValue(inpts[4], operations)

    variance = var + eps
    scale = np.reciprocal(np.sqrt(variance)) * scale_param
    bias = offset - (mean * scale)

    x.loadMultiplier(scale)
    x.loadBias(bias)

    return [x]
